"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const httpResponse_1 = require("./httpResponse");
const Error_1 = require("./Error");
/**
 * Clase base para los Worker y Fork
 */
class Base {
    /**
     * Contructor de la clase
     * @param conf Configuracion del servicio
     */
    constructor(conf, topic, span) {
        /**
         * Topico del servicio
         * @protected
         */
        this.Topic = '';
        this.config = conf;
        this.Topic = topic;
        this.span = span;
    }
    /**
     * Metodo para dar formato a la metadata de http
     *
     * @param {DataRequest} data
     * @param {(any | HttpResponse)} resp
     * @returns {*}
     * @memberof Base
     */
    httpFormat(data, resp) {
        if (resp instanceof httpResponse_1.HttpResponse) {
            // Agregar los elementos de la metadata
            data.metadata.http = {
                code: resp.status,
                headers: resp.headers,
                message: resp.message,
            };
            // Retornar la respuesta con formato para el procesado
            return { request: data, data: resp.data };
        }
        else {
            return { request: data, data: resp };
        }
    }
    /**
     * Generador del formato de error
     *
     * @param {DataRequest} data
     * @param {*} error
     * @param {*} ff
     * @memberof Base
     */
    errorHandler(data, error, ff) {
        if (error instanceof Error_1.MsError) {
            const e = error;
            const format = this.httpFormat(data, new httpResponse_1.HttpResponse(null, e.status, e.message, e.headers));
            // Crear la salida
            const r = this.formatResponse(format.request, format.data, error.code);
            ff(r);
        }
        else {
            console.log(error);
            // Regresar el error
            ff(this.formatResponse(data, null, error.code));
        }
    }
    /**
     * Metodo que hace rel registro de Forks
     * para su utilizacion sobre los servicios.
     * @param conf Configuracion para el evento
     */
    forksConfig(conf) {
        return;
    }
    transformFilters(filters) {
        if (util_1.isNull(filters)) {
            return filters;
        }
        else if (util_1.isObject(filters)) {
            const r = {};
            // tslint:disable-next-line: forin
            for (const k in filters) {
                let val = null;
                if (util_1.isArray(filters[k])) {
                    val = filters[k][0];
                }
                else {
                    val = filters[k];
                }
                r[k] = val;
            }
            return r;
        }
    }
    /**
     * Metodo para dar formato a los mensajes de salida.
     *
     * @param {DataRequest} Data Data que se pasa como resultado de la operacion
     * @param {*} response Respuesta
     * @param {number} errorCode Codigo del error al momento de la excepcion
     * @param {boolean} isForkError Indicador si el error proviene de una bifurcacion.
     * @param {ErrorMetadata} errorMeta Metadata del error de la bifurcacion.
     * @returns {ResponseMessage}
     * @memberof Base
     */
    formatResponse(Data, response, errorCode = null, isForkError = false, errorMeta) {
        // Variable para conocer si es un error o no
        let isError = false;
        // Validar si es un error
        if (!util_1.isNull(errorCode) && errorCode < 0) {
            isError = true;
        }
        // Objecto base de respuesta
        const resp = {
            data: Data.data,
            headers: Data.headers,
            metadata: Data.metadata,
            response: {
                data: {
                    response,
                },
                meta: {
                    id_transaction: Data.metadata.id_transaction,
                    status: isError ? 'ERROR' : 'SUCCESS',
                },
            },
            uuid: Data.uuid,
        };
        // Validar si es un error desde una bifurcacion.
        if (isForkError) {
            // Regresar todo
            return {
                error: errorMeta,
                fork: isForkError,
                response: resp,
            };
        }
        else {
            // Regresar todo
            return {
                error: {
                    errorCode,
                    userMessage: '',
                },
                fork: isForkError,
                response: resp,
            };
        }
    }
}
exports.default = Base;
